package server

import (
	"database/sql"
	"io/ioutil"
	"log"
	"muj-server2/db"
	"net/http"
	"strconv"

	"github.com/go-chi/chi"
	"github.com/go-chi/chi/middleware"
	"github.com/go-chi/render"
)

type Server struct {
	db     *db.DB
	listen string
}

// New creates new Server object
func New(db *db.DB, listen string) *Server {
	return &Server{
		db:     db,
		listen: listen,
	}
}

// Start the server
func (s *Server) Start() error {
	r := chi.NewRouter()

	r.Use(middleware.Logger)

	r.Route("/channel/{channel}/", func(r chi.Router) {
		r.Get("/", s.listMessages)
		r.Post("/", s.addMessage)
		r.Get("/{id}/", s.getMessage)
	})

	r.Route("/s-channel/{channel}/", func(r chi.Router) {
		r.Use(middleware.BasicAuth("Zadej heslo", map[string]string{"ksp": "hroch"}))

		r.Get("/", s.listMessages)
		r.Post("/", s.addMessage)
		r.Get("/{id}/", s.getMessage)
		r.Delete("/{id}/", s.deleteMessage)
	})

	log.Printf("Starting server on address %s", s.listen)
	return http.ListenAndServe(s.listen, r)
}

func (s *Server) listMessages(w http.ResponseWriter, r *http.Request) {
	channel := chi.URLParam(r, "channel")
	messages, err := s.db.GetAllMessages(channel)
	if err != nil {
		http.Error(w, err.Error(), http.StatusInternalServerError)
	} else {
		render.JSON(w, r, messages)
	}
}

func (s *Server) addMessage(w http.ResponseWriter, r *http.Request) {
	channel := chi.URLParam(r, "channel")
	body, err := ioutil.ReadAll(r.Body)
	if err != nil {
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}

	if err := s.db.SaveMessage(channel, string(body)); err != nil {
		http.Error(w, err.Error(), http.StatusInternalServerError)
	} else {
		w.WriteHeader(http.StatusOK)
	}
}

func (s *Server) getMessage(w http.ResponseWriter, r *http.Request) {
	channel := chi.URLParam(r, "channel")
	id, err := strconv.Atoi(chi.URLParam(r, "id"))
	if err != nil {
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}

	msg, err := s.db.GetMessage(id)
	if err == sql.ErrNoRows || msg.Channel != channel {
		http.Error(w, err.Error(), http.StatusNotFound)
	} else {
		render.JSON(w, r, msg)
	}
}

func (s *Server) deleteMessage(w http.ResponseWriter, r *http.Request) {
	channel := chi.URLParam(r, "channel")
	id, err := strconv.Atoi(chi.URLParam(r, "id"))
	if err != nil {
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}

	msg, err := s.db.GetMessage(id)
	if err == sql.ErrNoRows || msg.Channel != channel {
		http.Error(w, err.Error(), http.StatusNotFound)
		return
	}

	if err := s.db.DeleteMessage(id); err != nil {
		http.Error(w, err.Error(), http.StatusInternalServerError)
	} else {
		w.WriteHeader(http.StatusOK)
	}
}
